@@ -1,3 +1,5 @@
+{-# LANGUAGE LambdaCase #-}
+
 {- |
    Module      : Distribution.Gentoo.PkgManager
    Description : Using package managers in Gentoo.
@@ -14,17 +16,21 @@ module Distribution.Gentoo.PkgManager
        , defaultPM
        , defaultPMName
        , nameOfPM
+       , toPkgManager
        , buildCmd
        , buildAltCmd
        ) where
 
 import Distribution.Gentoo.Packages
 import Distribution.Gentoo.PkgManager.Types
+import Distribution.Gentoo.Types
+import qualified Distribution.Gentoo.Types.HUMode as Mode
 
 import Data.Char(toLower)
 import Data.Maybe(mapMaybe, fromMaybe)
 import qualified Data.Map as M
 import Data.Map(Map)
+import qualified Data.Set as Set
 import System.Environment(getEnvironment)
 
 -- -----------------------------------------------------------------------------
@@ -87,7 +93,6 @@ defaultPMFlags               :: PkgManager -> [String]
 defaultPMFlags Portage       = [ "--oneshot"
                                , "--keep-going"
                                , "--complete-graph"
-                               , "--usepkg=n"
                                ]
 defaultPMFlags PkgCore       = [ "--deep"
                                , "--oneshot"
@@ -101,11 +106,35 @@ defaultPMFlags Paludis       = [ "resolve"
 defaultPMFlags CustomPM{}    = []
 defaultPMFlags (InvalidPM _) = undefined
 
-buildCmd :: PkgManager -> [PMFlag] -> [String] -> [Package] -> (String, [String])
-buildCmd pm fs raw_pm_flags ps = (pmCommand pm, fs' ++ ps')
-    where
-      fs' = defaultPMFlags pm ++ mapMaybe (flagRep pm) fs ++ raw_pm_flags
-      ps' = map printPkg ps
+toPkgManager :: Mode.PkgManager -> PkgManager
+toPkgManager (Mode.Portage _) = Portage
+toPkgManager (Mode.PkgCore _) = PkgCore
+toPkgManager (Mode.Paludis _) = Paludis
+toPkgManager (Mode.CustomPM s _) = CustomPM s
+
+buildCmd
+    :: Mode.PkgManager
+    -> [PMFlag]
+    -> [String]
+    -> DefaultModePkgs
+    -> (String, [String])
+buildCmd mpm fs raw_pm_flags ps =
+    (  pmCommand pm
+    ,  defaultPMFlags pm
+    ++ mapMaybe (flagRep pm) fs
+    ++ raw_pm_flags
+    ++ rest
+    )
+  where
+    rest = case (pm, ps) of
+        (Portage, DefaultInvalid p) -> usepkgExclude p ++ targs p
+        (Portage, DefaultAll a) -> usepkgExclude a ++ targs a
+        (_, DefaultInvalid p) -> targs p
+        (_, DefaultAll a) -> targs a
+
+    targs p = printPkg <$> Set.toList (getPkgs p)
+
+    pm = toPkgManager mpm
 
 -- | Alternative version of 'buildCmd' which uses experimental @emerge@
 --   invocation (using @--reinstall-atoms@). This is only to be used with the
@@ -121,16 +150,50 @@ buildCmd pm fs raw_pm_flags ps = (pmCommand pm, fs' ++ ps')
 buildAltCmd
     :: [PMFlag] -- ^ Basic flags
     -> [String] -- ^ User-supplied flags
-    -> [Package] -- ^ List of packages to rebuild
-    -- | List of /all/ installed haskell packages ('Nothing' denotes a @world@ target)
-    -> Maybe [Package]
+    -> RAModePkgs -- ^ Set of packages to rebuild
+    -- | 'True' denotes a @world@ target
+    -> AllPkgs
     -> (String, [String])
-buildAltCmd fs rawPmFlags ps allPs =
-    (pmCommand Portage, fs' ++ reinst ++ rawPmFlags ++ allPs')
+buildAltCmd fs rawPmFlags raPS allPs =
+    (  pmCommand Portage
+    ,  defaultPMFlags Portage
+    ++ mapMaybe (flagRep Portage) fs
+    ++ ["--update"]
+    ++ rawPmFlags
+    ++ usepkgExclude allPs
+    ++ reinst
+    ++ targs
+    )
   where
-    fs' = defaultPMFlags Portage ++ mapMaybe (flagRep Portage) fs ++ ["--update"]
-    reinst = ["--reinstall-atoms", unwords (map printPkg ps)]
-    allPs' = maybe ["@world"] (map printPkg) allPs
+    (reinst, targs) =
+        let raArgs ps
+              | Set.null ps = []
+              | otherwise = ["--reinstall-atoms", unwords (printPkg <$> Set.toList ps)]
+        in case raPS of
+            RAModeInvalid p ->
+                (raArgs (getPkgs p), printPkg <$> Set.toList (getPkgs allPs))
+            RAModeAll ->
+                (raArgs (getPkgs allPs), printPkg <$> Set.toList (getPkgs allPs))
+            RAModeWorld p ->
+                (raArgs (getPkgs p), ["@world"])
+            RAModeCustom ts p ->
+                (raArgs (getPkgs p), ts)
+
+-- | Generate strings using portage's @--usepkg-exclude@ flag. This filters out
+--   dev-haskell/* packages which can be specified using a wildcard, in order
+--   to reduce the length of the emerge command a bit.
+usepkgExclude :: PackageSet t => t -> [String]
+usepkgExclude pkgs0
+    | Set.null pkgs = []
+    | otherwise = ["--usepkg-exclude", unwords ("dev-haskell/*" : filteredPkgs)]
+  where
+    filteredPkgs = mapMaybe
+        ( \case
+              Package "dev-haskell" _ _ -> Nothing
+              p -> Just $ printPkg p
+        )
+        (Set.toList pkgs)
+    pkgs = getPkgs pkgs0
 
 -- -----------------------------------------------------------------------------
 
